       
        /*   
         Arduino code for simple dashboard with TM1638
        */

        // Serial parser for Interface - Output : M<Dash2>S<Dash1>G<Dash3>
        // Serial parser for Shutdown - Output : Z00
        // M : rpm              (16bit - hexadecimal output)
        // S : speed            (16bit - hexadecimal output)
        // G : gear             (16bit - hexadecimal output)
        // Z : clear            (to clear display and max rpm value)
        //
        // for dashboard
        // Pin D3 : STB0 - strobe
        // Pin D4 : CLK - clock
        // Pin D5 : DIO - data

        ////////////////////////////////////////////////////////////////////////////////
        // DashBoard include
        ////////////////////////////////////////////////////////////////////////////////   
        #include <TM1638.h> //can be downloaded from http://code.google.com/p/tm1638-library/
        // define a module on data pin 5, clock pin 4 and strobe pin 3
        TM1638 module(5, 4, 3);
       
        int i;
        unsigned int carspeed;      // holds the speed data (0-65535 size)
        unsigned int gear;          // holds gear value data (0-65535 size)
        unsigned int rpm;           // holds the rpm data (0-65535 size)
        unsigned int rpmleds;       // holds the 8 leds values
        unsigned int rpmmax = 5000; // retrieves from rpm live with minimum setting on initialization and Shutdown - Output
        byte speeddata = 0;         // marker that new data are available
        byte geardata = 0;          // marker that new data are available   
        byte rpmdata = 0;           // marker that new data are available
        // char* neutral = "n";        // sets the character for neutral
        char* neutral = "0";        // sets the character for neutral
        char* reverse = "r";        // sets the character for reverse
           
        String namesz = "88888888"; // sets a custom logo start up banner
       
        ////////////////////////////////////////////////////////////////////////////////
        // INITIALIZATION
        ////////////////////////////////////////////////////////////////////////////////
        void setup()
        {
          // serial initialization
          Serial.begin(115200);
         
          // initialize the screen with leds test
          module.clearDisplay();                       // clears the display from garbage if any
          module.setDisplayToString(namesz);           // prints the banner
          delay(1000);                                 // small delay 1 sec
          module.clearDisplay();                       // clears the display
          module.setLEDs(0b11111111 | 0b00000000<< 8); // all green leds
          delay(1000);                                 // small delay 1 sec
          module.setLEDs(0b00000000 | 0b11111111<< 8); // all reds leds
          delay(1000);                                 // small delay 1 sec
          module.setLEDs(0b00000000 | 0b00000000<< 8); // turn off all leds
        }
       
        ////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////// Main Loop ////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////
        void loop()
        {

          // read input data
          readSerialData();

          // update dashboard
          dashboard();

        }
       
        ////////////////////////////////////////////////////////////////////////////////
        // Procedure: wait for complete trame
        ////////////////////////////////////////////////////////////////////////////////
        void readSerialData()
        {
          byte Data[5]={'0','0','0','0','0'};

          // parse the buffer
          if (Serial.available()>4){
            Data[0]=Serial.read();                   // read command
            if (Data[0]=='M'){                       // if rpm data
              Data[1]=Serial.read();                 // store high byte
              Data[2]=Serial.read();                 //
              Data[3]=Serial.read();                 //
              Data[4]=Serial.read();                 // store low byte
              rpm = NormalizeData(Data);
              rpmdata=1;                             // we got new data!       
              if (Serial.available()>4){
                Data[0]=Serial.read();               // re-read command
              }
            }
            if (Data[0] == 'S' ){                    // if speed data
              Data[1]=Serial.read();                 // store high byte
              Data[2]=Serial.read();                 //
              Data[3]=Serial.read();                 //
              Data[4]=Serial.read();                 // store low byte
              carspeed = NormalizeData(Data);
              speeddata=1;                           // we got new data!
              if (Serial.available()>4){
                Data[0]=Serial.read();               // re-read command
              }
            }
            if (Data[0] == 'G' ){                    // if gear data
              Data[1]=Serial.read();                 // store high byte
              Data[2]=Serial.read();                 //
              Data[3]=Serial.read();                 //
              Data[4]=Serial.read();                 // store low byte
              gear = NormalizeData(Data);
              geardata=1;                            // we got new data!
            }             
            if (Data[0] == 'Z' ){                          // if Shutdown - Output
              Serial.flush();
              rpmmax=5000;                                 // reset rpmmax
              module.clearDisplay();                       // clear display   
              module.setLEDs(0b00000000 | 0b00000000<< 8); // clear leds
            }             
          }
          if (Serial.available()>40) Serial.flush();
        }

        ////////////////////////////////////////////////////////
        void dashboard()
        ////////////////////////////////////////////////////////
        {   
          if (speeddata == 1) {
            // filter of negative value
            // reported from 65535 to down in some games
            if (carspeed > 50000){                            // reverse velocity in some games are negative value
              carspeed = 65536 - carspeed;                    // convert positive with 16bit value
            }
            module.setDisplayToDecNumber(carspeed, 0, false); // displays numerical the speed
            speeddata=0;           
          } 
          if (geardata == 1) {
            if (gear >= 1 and gear <10 ){         
              module.setDisplayDigit(gear, 0, false);         // displays numerical value of the current gear
            }
            if (gear == 0){
              module.setDisplayToString(neutral, 0, 0);       // displays the character for neutral
            }
            if (gear == (65535)){                             // -1 that reprecents reverse rollover so...
              module.setDisplayToString(reverse, 0, 0);       // displays the character for reverse
            }
            geardata=0;
          } 
          if (rpmdata == 1) {           
            // auto set max rpm
            // with filter of negative value
            // reported from 65535 to down in some games
            if (rpm>rpmmax && rpm<50000) {
              rpmmax=rpm;
            }
            rpmleds = map(rpm,0,rpmmax*1.1,0,9);              // distributes the rpm level to the 8 leds + 1 for shift change
            if (rpmleds==0){
              module.setLEDs(0b00000000 | 0b00000000<< 8);
            }
            if (rpmleds==1){
              module.setLEDs(0b00000001 | 0b00000000<< 8 );
            }
            if (rpmleds==2){
              module.setLEDs(0b00000011 | 0b00000000<< 8 );
            }
            if (rpmleds==3){
              module.setLEDs(0b00000111 | 0b00000000<< 8 );
            }
            if (rpmleds==4){
              module.setLEDs(0b00001111 | 0b00000000<< 8);
            }
            if (rpmleds==5){
              module.setLEDs(0b00011111 | 0b00000000<< 8);
            }
            if (rpmleds==6){
              module.setLEDs(0b00011111 | 0b00100000<< 8 );
            }
            if (rpmleds==7){
              module.setLEDs(0b00011111 | 0b01100000<< 8 );
            }
            if (rpmleds==8){
              module.setLEDs(0b11111111 | 0b1111111111<< 8 );
            }
            rpmdata=0;
          }
        }
         
        ////////////////////////////////////////////////////////////////////////////////
        // Function: convert Hex to Dec
        ////////////////////////////////////////////////////////////////////////////////
        unsigned int NormalizeData(byte x[5])
        ////////////////////////////////////////////////////////////////////////////////
        {
        unsigned int result;
  
        for (int i=1; i<5; i++)
        {
          if (x[i]>47 && x[i]<58 ){   //for xA to xF
            x[i]=x[i]-48;
          }                       
          if (  x[i]>64 && x[i]<71 ){ //for x0 to x9
            x[i]=x[i]-55;               
          }
        }    
        result = (x[1]*16*16*16+x[2]*16*16+x[3]*16+x[4]);
        return result;
        }
        
